--
-- Main file of the Dungeon Locations addon
--
-- Author: staratnight
-- Date: 31.12.2017
-- Copyright 2017 - Torsten Mingers
--
-- Licensed under the terms of the GNU General Public License (Version 3).
-- You should have received a copy of the GNU General Public License
-- along with Dungeon Locations. If not, see <http://www.gnu.org/licenses/gpl-3.0.txt>.
--
-- See https://wow.curseforge.com/projects/dungeon-locations/issues to provide bugs or improvements.

local DungeonLocations, DLoc = ...
local L = DLoc_Locals

local currentMapId;
local iconScale = 24;

local EventListener = CreateFrame("Frame", "Helper");
EventListener:RegisterEvent("ZONE_CHANGED_NEW_AREA");
EventListener:RegisterEvent("ADDON_LOADED");
EventListener:RegisterEvent("WORLD_MAP_UPDATE");
EventListener:RegisterEvent("WORLD_MAP_NAME_UPDATE");

local function eventHandler(_, event, arg1)
    if (event == "ADDON_LOADED" and arg1 == "DungeonLocations") then
		DLoc:MapTooltipSetup();
        DLoc:CreateLocations();
        DLoc:CreateWorldMapLocations()
        DLoc:AddChatMessage(L.LoadingSuccessMessage);
    end
    if(event ~= "ADDON_LOADED") then
        local mapId = GetCurrentMapAreaID()
        DLoc:HideAllDungeonLocations();
        DLoc:ShowDungeonLocations(mapId);
        DLoc:ShowWorldMapDungeonLocations(mapId);
    end
end

EventListener:SetScript("OnEvent", eventHandler);

 function DLoc:CreateLocation(x, y, dungeon)
     local dungeonLocation = CreateFrame("Frame", "Dungeon", WorldMapDetailFrame)
	 
     dungeonLocation.Texture = dungeonLocation:CreateTexture();
     if(dungeon.dungeonType == Type_Dungeon) then
        dungeonLocation.Texture:SetTexture('Interface\\Addons\\DungeonLocations\\Art\\portal_dungeon')
     else
        dungeonLocation.Texture:SetTexture('Interface\\Addons\\DungeonLocations\\Art\\portal_raid')
     end

     dungeonLocation.Texture:SetAllPoints();
     dungeonLocation:EnableMouse(true);
     dungeonLocation:SetFrameStrata("TOOLTIP");
     dungeonLocation:SetFrameLevel(WorldMapDetailFrame:GetFrameLevel() + 10);

     dungeonLocation:SetPoint("CENTER", WorldMapDetailFrame, "TOPLEFT", (x / 100) * WorldMapDetailFrame:GetWidth(), (-y / 100) * WorldMapDetailFrame:GetHeight());

     dungeonLocation:SetWidth(iconScale);
     dungeonLocation:SetHeight(iconScale);

     dungeonLocation:HookScript("OnEnter",
         function(pin)
             DLoc_MapTooltip:SetOwner(pin, "ANCHOR_RIGHT")
             DLoc_MapTooltip:SetScale(WorldMapFrame:GetScale())
			 DLoc:BuildInfoTooltip(DLoc_MapTooltip, dungeon)
             DLoc_MapTooltip:Show()
         end
    )
     dungeonLocation:HookScript("OnLeave",
         function()
             DLoc_MapTooltip:Hide()
         end
    )
	dungeonLocation:HookScript("OnMouseDown",
         function()
			if ( not EncounterJournal ) then
				EncounterJournal_LoadUI();
			end
            local _, _, _, _, _, _, _, link = EJ_GetInstanceInfo(dungeon.id)
            local difficulty = string.match(link, 'journal:.-:.-:(.-)|h')
            EncounterJournal_OpenJournal(difficulty, dungeon.id)
         end
    )
     dungeonLocation:Hide()
     return dungeonLocation
 end

 function DLoc:CreateWorldMapLocation(x, y, dungeons)
    local dungeonLocation = CreateFrame("Frame", "Dungeons", WorldMapDetailFrame)
    
    dungeonLocation.Texture = dungeonLocation:CreateTexture();
    dungeonLocation.Texture:SetTexture('Interface\\Addons\\DungeonLocations\\Art\\portal_dungeon')

    for i = 1, #dungeons do
        if(dungeons[i].dungeonType == Type_Raid) then
            dungeonLocation.Texture:SetTexture('Interface\\Addons\\DungeonLocations\\Art\\portal_raid')
            break
        end
    end

    dungeonLocation.Texture:SetAllPoints();
    dungeonLocation:EnableMouse(true);
    dungeonLocation:SetFrameStrata("TOOLTIP");
    dungeonLocation:SetFrameLevel(WorldMapDetailFrame:GetFrameLevel() + 10);

    dungeonLocation:SetPoint("CENTER", WorldMapDetailFrame, "TOPLEFT", (x / 100) * WorldMapDetailFrame:GetWidth(), (-y / 100) * WorldMapDetailFrame:GetHeight());

    dungeonLocation:SetWidth(iconScale);
    dungeonLocation:SetHeight(iconScale);

    dungeonLocation:HookScript("OnEnter",
        function(pin)
            DLoc_MapTooltip:SetOwner(pin, "ANCHOR_RIGHT")
            DLoc_MapTooltip:SetScale(WorldMapFrame:GetScale())
            DLoc:BuildWorldMapInfoTooltip(DLoc_MapTooltip, dungeons)
            DLoc_MapTooltip:Show()
        end
   )
   dungeonLocation:HookScript("OnLeave",
        function()
            DLoc_MapTooltip:Hide()
        end
   )
   dungeonLocation:Hide()
   return dungeonLocation
end

 function DLoc:HideAllDungeonLocations()
    if(currentMapId == nil) then
        return;
    end
    
    if(DLoc:isWorldMapId(currentMapId)) then
        local locations = DLoc:getWorldMapDungeonLocations(currentMapId)
        for i = 1, #locations do
            locations[i]:Hide();
        end
    end
    if(DLoc:hasId(currentMapId)) then
        local locations = DLoc:getDungeonLocations()	 
        for i = 1, #locations[currentMapId] do
            locations[currentMapId][i]:Hide();
        end
    end
    currentMapId = nil;
 end

function DLoc:ShowDungeonLocations(worldMapId)
    if(DLoc:hasId(worldMapId) == false) then
        return
    end

	local realWorldMapId = DLoc:DetermineAreaIdForSpecialMapAreas(worldMapId);
	if(realWorldMapId == nil) then
		return
	end
	 
    local locations = DLoc:getDungeonLocations()
    for i = 1, #locations[worldMapId] do
        locations[worldMapId][i]:Show();
    end
    currentMapId = worldMapId;

 end
 
function DLoc:ShowWorldMapDungeonLocations(worldMapId)
    if(DLoc:isWorldMapId(worldMapId) == false) then
        return
    end

    local locations = DLoc:getWorldMapDungeonLocations(worldMapId)
    for i = 1, #locations do
        locations[i]:Show();
    end
    currentMapId = worldMapId;
end

function DLoc:MapTooltipSetup()
    DLoc_MapTooltip = CreateFrame("GameTooltip", "DLoc_MapTooltip", WorldMapFrame, "GameTooltipTemplate")
    DLoc_MapTooltip:SetFrameStrata("TOOLTIP")
    DLoc_MapTooltip:SetFrameLevel(WorldMapDetailFrame:GetFrameLevel() + 20)
    WorldMapFrame:HookScript("OnSizeChanged",
        function(self)
            DLoc_MapTooltip:SetScale(1 / self:GetScale())
        end
    )
end

function DLoc:BuildInfoTooltip(DLoc_MapTooltip, dungeon)
	DLoc_MapTooltip:ClearLines()
	DLoc_MapTooltip:AddDoubleLine(dungeon.name, dungeon.level, 1, 1, 1, false)
	
	local difficulty = L.DiffNormal
	if(dungeon.heroic) then
		difficulty = string.format("%s/%s", difficulty, L.DiffHero)
	end
	if(dungeon.mythic) then
		difficulty = string.format("%s/%s", difficulty, L.DiffMythic)
	end
	DLoc_MapTooltip:AddLine(difficulty, 1, 1, 1, false)
		
	if(dungeon.dungeonType == Type_Raid) then
		players = string.format("%s: %s", L.Players, dungeon.player)
		DLoc_MapTooltip:AddLine(players, 1, 1, 1, false)
	end
	
	if(dungeon.additionalInfo) then
		DLoc_MapTooltip:AddLine(dungeon.additionalInfo, nil, nil, nil, true)
	end
end

function DLoc:BuildWorldMapInfoTooltip(DLoc_MapTooltip, dungeons)
	DLoc_MapTooltip:ClearLines()
	DLoc_MapTooltip:AddLine(L.DungeonsInThisArea, 1, 1, 1, false)
    
    for i = 1, #dungeons do
        local type
        if(dungeons[i].dungeonType == Type_Raid) then
            type = L.TypeRaid
        else
            type = L.TypeDungeon
        end
        DLoc_MapTooltip:AddDoubleLine(dungeons[i].name, type .. " (" .. dungeons[i].level .. ")", 1, 1, 1, false)
    end
end

function DLoc:AddChatMessage(message)
    DEFAULT_CHAT_FRAME:AddMessage(message, 1, 1, 0);
end